// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include "fw/base.h"
#include "fw/dynmx/base.h"

#include <schpin_lib_exp/protocol/register_map.h>

#pragma once

namespace schpin
{
enum dynmx_config_enum : uint8_t
{
        DYNMX_INPUT_RANGE_LOW  = 0,
        DYNMX_INPUT_RANGE_HIGH = 1,
        DYNMX_SERVO_RANGE_LOW  = 2,
        DYNMX_SERVO_RANGE_HIGH = 3
};

using dynmx_config_map_def = protocol_map<
    dynmx_config_enum,
    protocol_reg< DYNMX_INPUT_RANGE_LOW, bangle >,
    protocol_reg< DYNMX_INPUT_RANGE_HIGH, bangle >,
    protocol_reg< DYNMX_SERVO_RANGE_LOW, dynmx_angle >,
    protocol_reg< DYNMX_SERVO_RANGE_HIGH, dynmx_angle > >;

struct dynmx_config_map : dynmx_config_map_def
{
        using dynmx_config_map_def::protocol_map;

        dynmx_config_map()
          : dynmx_config_map_def(
                bangle::from_float( 0.f ),
                bangle::from_float( 4.18879f ),
                dynmx_angle::get< 0 >(),
                dynmx_angle::get< 4095 >() )
        {
        }
};

}  // namespace schpin
