// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include "fw/dynmx/protocol.h"
#include "fw/serial.h"

#include <schpin_lib_exp/protocol.h>

namespace schpin
{

struct dynmx_serial
{
        serial_connection                                     ser;
        std::array< uint8_t, dynmx_master_message::max_size > buffer;

        dynmx_serial( serial_connection conn )
          : ser( conn )
        {
        }

        void send( const master_dynmx_message& m )
        {
                ser.write( m );
        }

        void log_error()
        {
                SCHPIN_ERROR_LOG(
                    "s", "Error " << errno << " when console: " << strerror( errno ) );
        }

        dynmx_master_message read_reply()
        {
                std::size_t wanted = 7;
                std::size_t got    = 0;

                while ( got != wanted ) {
                        ssize_t n = ser.read( buffer.begin() + got, wanted - got );
                        if ( n < 0 ) {
                                log_error();
                                continue;
                        }
                        got += static_cast< std::size_t >( n );
                        if ( got < 7 ) {
                                continue;
                        }
                        protocol_item< dynmx_packet_size >::deserialize(
                            em::view_n( buffer.begin() + 5, 2 ),
                            match(
                                [&]( error_tag, auto ) {
                                        SCHPIN_ERROR_LOG(
                                            "dynmx.protocol", "Parsed packet length is invalid" );
                                },
                                [&]( value_tag, dynmx_packet_size val ) {
                                        wanted += *val;
                                } ) );
                }

                return dynmx_master_message{ buffer, got };
        }
};

}  // namespace schpin
