// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include <schpin_lib_exp/protocol/register_map.h>

#pragma once

namespace schpin
{

enum lewan_config_enum : uint8_t
{
        LEWAN_INPUT_RANGE_LOW     = 0,
        LEWAN_INPUT_RANGE_HIGH    = 1,
        LEWAN_SERVO_RANGE_LOW     = 2,
        LEWAN_SERVO_RANGE_HIGH    = 3,
        LEWAN_ANGLE_CHANGE_REPORT = 4,
        LEWAN_ANGLE_STALL_RANGE   = 5,
        LEWAN_GOTO_P_GAIN         = 6,
        LEWAN_GOTO_I_GAIN         = 7,
        LEWAN_GOTO_D_GAIN         = 8,
        LEWAN_VEL_P_GAIN          = 9,
        LEWAN_VEL_I_GAIN          = 10,
        LEWAN_VEL_D_GAIN          = 11
};

using lewan_config_map_def = protocol_map<
    lewan_config_enum,
    protocol_reg< LEWAN_INPUT_RANGE_LOW, bangle >,
    protocol_reg< LEWAN_INPUT_RANGE_HIGH, bangle >,
    protocol_reg< LEWAN_SERVO_RANGE_LOW, lewan_angle >,
    protocol_reg< LEWAN_SERVO_RANGE_HIGH, lewan_angle >,
    protocol_reg< LEWAN_ANGLE_CHANGE_REPORT, lewan_angle >,
    protocol_reg< LEWAN_ANGLE_STALL_RANGE, lewan_angle >,
    protocol_reg< LEWAN_GOTO_P_GAIN, pid_gain >,
    protocol_reg< LEWAN_GOTO_I_GAIN, pid_gain >,
    protocol_reg< LEWAN_GOTO_D_GAIN, pid_gain >,
    protocol_reg< LEWAN_VEL_P_GAIN, pid_gain >,
    protocol_reg< LEWAN_VEL_I_GAIN, pid_gain >,
    protocol_reg< LEWAN_VEL_D_GAIN, pid_gain > >;

struct lewan_config_map : lewan_config_map_def
{
        using lewan_config_map_def::protocol_map;

        lewan_config_map()
          : lewan_config_map_def(
                bangle::from_float( 0.f ),
                bangle::from_float( 4.18879f ),
                lewan_angle::get< 0 >(),
                lewan_angle::get< 1000 >(),
                lewan_angle::get< 5 >(),
                lewan_angle::get< 25 >(),
                pid_gain::from_float( 10.f ),
                pid_gain::from_float( 0.f ),
                pid_gain::from_float( 150.f ),
                pid_gain::from_float( 0.6f ),
                pid_gain::from_float( 0.005f ),
                pid_gain::from_float( 0.6f ) )
        {
        }
};

}  // namespace schpin
