// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include <schpin_lib_exp/protocol/base.h>
#include <tuple>

#pragma once

namespace schpin
{

template < auto... Ids >
class cmd_roulette
{
        std::tuple< cmd_tag< Ids >... > ids_tpl_{};
        std::size_t                     i_   = 0;
        static constexpr std::size_t    size = sizeof...( Ids );

      public:
        std::size_t get_size() const
        {
                return size;
        }
        std::size_t get_index() const
        {
                return i_;
        }
        template < typename Function >
        void tick( Function&& f )
        {
                std::size_t i = 0;
                em::for_each( ids_tpl_, [&]( auto tag ) {
                        if ( i == i_ ) {
                                f( tag );
                        }
                        i += 1;
                } );

                i_ = ( i_ + 1 ) % size;
        }
};

}  // namespace schpin
