// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include "fw/reactor.h"
#include "stm32g4xx_hal.h"

using namespace schpin;

using reactor = fw_reactor< 20 >;

UART_HandleTypeDef master_uart;
UART_HandleTypeDef lewan_uart;

reactor rec;

clk_time get_systime()
{
        return clk_time{ HAL_GetTick() };
}

struct stm32_commands
{

        void operator()( control_tag, const fw_node_group_message& group_msg )
        {
                auto m = fw_node_protocol::serialize( ctl_id{ 0 }, group_msg, checksum{} );
                HAL_UART_Transmit( &master_uart,
                                   const_cast< uint8_t* >( m.begin() ),
                                   static_cast< uint16_t >( m.size() ),
                                   200 );
        }
        void operator()( servo_tag, s_ctl_id msg_id, reply r, const master_lewan_message& m )
        {
                HAL_UART_Transmit( &lewan_uart,
                                   const_cast< uint8_t* >( m.begin() ),
                                   static_cast< uint16_t >( m.size() ),
                                   200 );
                if ( r == reply::NO ) {
                        return;
                }

                std::array< uint8_t, 10 > data;

                HAL_UART_Receive( &master_uart,
                                  const_cast< uint8_t* >( data.begin() ),
                                  static_cast< uint16_t >( data.size() ),
                                  200 );

                rec.on_servo_receive( get_systime(), msg_id, data, stm32_commands{} );
        }
};

int main()
{

        while ( 1 ) {
                rec.tick( get_systime(), stm32_commands{} );
        }
}
