// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include "emlabcpp/physical_quantity.h"
#include "emlabcpp/quantity.h"

#include <cmath>
#include <limits>
#include <ostream>
#include <ratio>
#include <type_traits>

#ifdef SCHPIN_USE_NLOHMANN_JSON
#include <nlohmann/json.hpp>
#endif

#pragma once

namespace em = emlabcpp;

#ifdef SCHPIN_USE_NLOHMANN_JSON
// In case nlohmann/json.hpp is present, provides abillity to serialize quantities to json.
//
// Quantities that have Tag::get_unit() nonequal to "" are serialized as: {Tag::get_unit() :
// *quantity}
template < typename T, typename ValueType >
struct nlohmann::adl_serializer< em::quantity< T, ValueType > >
{
        static void to_json( nlohmann::json& json, const em::quantity< T, ValueType > quantity )
        {
                if ( T::get_unit().empty() ) {
                        json = *quantity;
                } else {
                        json[T::get_unit()] = *quantity;
                }
        }
        static T from_json( const nlohmann::json& j )
        {
                if ( T::get_unit().empty() ) {
                        return T{ j.get< ValueType >() };
                }
                return T{ j[T::get_unit()].template get< ValueType >() };
        }
};

template < int Len, int Mass, int Time, int Current, int Temp, int Mol, int Li, int Angle, int Byte >
struct nlohmann::adl_serializer<
    emlabcpp::physical_quantity< Len, Mass, Time, Current, Temp, Mol, Li, Angle, Byte > >
  : nlohmann::adl_serializer< emlabcpp::quantity<
        emlabcpp::physical_quantity< Len, Mass, Time, Current, Temp, Mol, Li, Angle, Byte >,
        float > >
{
};

#endif

namespace emlabcpp
{
template < typename T, typename ValueType >
inline std::ostream& operator<<( std::ostream& os, quantity< T, ValueType > q )
{
        return os << *q << T::get_unit();
}
}  // namespace emlabcpp
