// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include <emlabcpp/static_vector.h>
#include <schpin_lib_exp/protocol/message.h>

#pragma once

namespace em = emlabcpp;  // tODO: these are randomly but around the codebase, fix!

namespace schpin
{

// TODO: copypasta from message :(

template < std::size_t N >
class prot_sizeless_message
{
        std::array< uint8_t, N > data_ = { 0 };
        std::size_t              used_ = 0;

        template < typename Iterator >
        prot_sizeless_message( Iterator begin, Iterator end )
          : used_( static_cast< std::size_t >( std::distance( begin, end ) ) )
        {
                std::copy( begin, end, data_.begin() );
        }

      public:
        using value_type     = uint8_t;
        using iterator       = uint8_t*;
        using const_iterator = const uint8_t*;

        static constexpr std::size_t max_size = N;

        template < typename Container >
        static std::optional< prot_sizeless_message > make( Container&& cont )
        {
                if ( cont.size() > N ) {
                        return {};
                }
                return { prot_sizeless_message( cont.begin(), cont.end() ) };
        }

        prot_sizeless_message() = default;

        // TODO: maybe make this explicit?
        template < std::size_t M >
        explicit prot_sizeless_message( prot_sizeless_message< M > other )
          : used_( other.size() )
        {
                static_assert( M <= N );
                std::copy( other.begin(), other.end(), data_.begin() );
        }

        template < std::size_t M >
        explicit prot_sizeless_message( const em::static_vector< uint8_t, M >& inpt ) noexcept
          : prot_sizeless_message( inpt.begin(), inpt.end() )
        {
        }

        template < std::size_t M >
        explicit prot_sizeless_message( const std::array< uint8_t, M >& inpt,
                                        std::size_t                     used = M ) noexcept
          : used_( used )
        {
                std::copy( inpt.begin(), inpt.end(), data_.begin() );
                static_assert( M <= N );
        }

        std::size_t size() const noexcept
        {
                return used_;
        }

        uint8_t* begin() noexcept
        {
                return data_.begin();
        }
        uint8_t* end() noexcept
        {
                return data_.begin() + used_;
        }
        const uint8_t* begin() const noexcept
        {
                return data_.begin();
        }
        const uint8_t* end() const noexcept
        {
                return data_.begin() + used_;
        }

        uint8_t& operator[]( std::size_t i ) noexcept
        {
                return data_[i];
        }

        uint8_t operator[]( std::size_t i ) const noexcept
        {
                return data_[i];
        }
};

}  // namespace schpin
