// Copyright 2021 Schpin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.


#include "base/physics.h"
#include "schpin_lib/geom/point.h"
#include "schpin_lib/view.h"
#include "util.h"

#pragma once

namespace schpin
{

enum class collision_mode
{
        FULL = 0,
        FAST = 1
};

template < typename PointIteratorT, typename SeparationAxisIterator >
class i_collision_item  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using point_iterator           = PointIteratorT;
        using separation_axis_iterator = SeparationAxisIterator;

        virtual em::view< point_iterator >           get_points() const            = 0;
        virtual em::view< separation_axis_iterator > get_separation_axes() const   = 0;
        virtual em::view< separation_axis_iterator > get_intersection_axes() const = 0;
        virtual ~i_collision_item()                                                = default;
};

template < typename FullT, std::size_t Nu >
class i_fast_collision_item
{
      public:
        using full_type                = FullT;
        static constexpr std::size_t n = Nu;
        using tag                      = typename full_type::tag;

        virtual point< n, tag > get_min_sphere_position() const = 0;
        virtual em::radius      get_min_sphere_radius() const   = 0;
        virtual full_type       get_full() const                = 0;
        virtual ~i_fast_collision_item()                        = default;
};

}  // namespace schpin
